searchData={"items":[{"type":"module","title":"KineticEcto","doc":"Documentation for `KineticEcto`.","ref":"KineticEcto.html"},{"type":"function","title":"KineticEcto.empty?/1","doc":"Returns true if the value would be considered empty.\n\n| value type    | result                             |\n| ------------- | ---------------------------------- |\n| `nil`         | `true`                             |\n| binary        | `String.trim_leading(value) == \"\"` |\n| list          | `value == []`                      |\n| tuple         | `tuple_size(value) == 0`           |\n| map           | `map_size(value) == 0`             |\n| Enumerable    | `Enum.empty?(value)`               |\n| anything else | `false`                            |\n\n- `nil` is empty\n- non-struct maps are empty if `map_size/1` is `0`\n- lists are empty if they equal `[]`\n- strings are empty if, when trimmed of leading spaces, they are equal to an\n  empty string (`\"\"`)\n- values implementing the `Enumerable` protocol are empty if `Enum.empty?/1`\n  returns `true`\n- all other values are not empty","ref":"KineticEcto.html#empty?/1"},{"type":"function","title":"Examples - KineticEcto.empty?/1","doc":"```elixir\niex> KineticEcto.empty?(nil)\ntrue\n\niex> KineticEcto.empty?(\"\")\ntrue\n\niex> KineticEcto.empty?([])\ntrue\n\niex> KineticEcto.empty?(\"   \")\ntrue\n\niex> KineticEcto.empty?(\" xyz \")\nfalse\n\niex> KineticEcto.empty?(MapSet.new([]))\ntrue\n\niex> KineticEcto.empty?([1])\nfalse\n\niex> KineticEcto.empty?(1..1//1)\nfalse\n\niex> KineticEcto.empty?(%{})\ntrue\n\niex> KineticEcto.empty?(%{a: 1})\nfalse\n\niex> KineticEcto.empty?({})\ntrue\n\niex> KineticEcto.empty?({1})\nfalse\n```","ref":"KineticEcto.html#empty?/1-examples"},{"type":"function","title":"KineticEcto.present?/1","doc":"Returns true if the value would not be considered empty. See `empty?/1` for\ndetails.","ref":"KineticEcto.html#present?/1"},{"type":"function","title":"Examples - KineticEcto.present?/1","doc":"```elixir\niex> KineticEcto.present?(nil)\nfalse\n\niex> KineticEcto.present?(\"\")\nfalse\n\niex> KineticEcto.present?([])\nfalse\n\niex> KineticEcto.present?(\"   \")\nfalse\n\niex> KineticEcto.present?(\" xyz \")\ntrue\n\niex> KineticEcto.present?(MapSet.new([]))\nfalse\n\niex> KineticEcto.present?([1])\ntrue\n\niex> KineticEcto.present?(1..1//1)\ntrue\n\niex> KineticEcto.present?(%{})\nfalse\n\niex> KineticEcto.present?(%{a: 1})\ntrue\n\niex> KineticEcto.present?({})\nfalse\n\niex> KineticEcto.present?({1})\ntrue\n```","ref":"KineticEcto.html#present?/1-examples"},{"type":"module","title":"KineticEcto.ChangesetValidations","doc":"Additional validations for use with Ecto.Changeset.","ref":"KineticEcto.ChangesetValidations.html"},{"type":"function","title":"KineticEcto.ChangesetValidations.validate_immutable/3","doc":"Validates that the field or fields provided are not changed once set.\n\nIf the data value (in the incoming struct) is present (not `nil` or blank), then any\nchange which contains an update to that value will be rejected.","ref":"KineticEcto.ChangesetValidations.html#validate_immutable/3"},{"type":"function","title":"KineticEcto.ChangesetValidations.validate_some_required/3","doc":"Validates that at least one of the `fields` is present (not `nil` or blank) in the\n`changeset`.\n\n`fields` must be a list of at least two schema field names.","ref":"KineticEcto.ChangesetValidations.html#validate_some_required/3"},{"type":"function","title":"Options - KineticEcto.ChangesetValidations.validate_some_required/3","doc":"- `:minimum`: The minimum number of fields that must have a value, default `1`. The\n  value provided will be clamped between `0` and the number of fields specified.\n- `:maximum`: The maximum number of fields that must have a value, default `nil`.\n  Unspecified by default. If specified, will be clamped between\n  specified, must be at least one and greater than or equal to `minimum` and will be\n  clamped to the number of fields specified.\n- `:minimum_message`: The message to display when the `minimum` threshold of presence is\n  not met.\n- `:maximum_message`: The message to display when the `maximum` threshold of presence is\n  exceeded.\n- `:exact_message`: The message to display when `minimum` and `maximum` are the same\n  exceeded.\n- `:message`: The message to be displayed if forwarded to `validate_required/2`.\n\nThere are two special cases:\n\n- If `minimum` is the number of fields passed, this calls `validate_required/2`.\n- If `minimum` is `0` and `maximum` is unspecified, this validation will always pass.\n\nOn validation failure, the `minimum_message` or `maximum_message` will be added to\n*each* field specified in `fields`.\n\nThe table below shows the special cases considered.\n\n| Minimum     | Maximum     | Result                               |\n| ----------- | ----------- | ------------------------------------ |\n| `0`         | `nil`       | pass                                 |\n| `#fields`   | -           | `Ecto.Changeset.validate_required/3` |\n| `< 0`       | -           | `minimum = 0`                        |\n| -           | `< 1`       | `maximum = 1`                        |\n| `> #fields` | -           | `minimum = #fields`                  |\n| -           | `> #fields` | `maximum = #fields`                  |\n| -           | `< minimum` | `maximum = minimum`                  |","ref":"KineticEcto.ChangesetValidations.html#validate_some_required/3-options"},{"type":"module","title":"KineticEcto.RepoTransact","doc":"Add Saša Jurić's `Repo.transact/2` to your repo with `use KineticEcto.RepoTransact`.\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.Postgres\n  use KineticEcto.RepoTransact\nend\n```\n\n`transact/2` is a replacement for [`Ecto.Repo.transaction/2`][3] with a better developer\nexperience. In many cases, the use of `transact/2` can provide code that is easier to\nunderstand than an equivalent implementation using `Ecto.Multi`.\n\nAs an example, a declarative user registration function might look like this example\nfrom Tom Konidas's [blog post][2]:\n\n```elixir\ndef register_user(params) do\n  Multi.new()\n  |> Multi.insert(:user, Accounts.new_user_changeset(params))\n  |> Multi.insert(:log, fn %{user: user} -> Logs.log_action(:user_registered, %{user: user}) end)\n  |> Multi.insert(:email_job, fn %{user: user} -> Mailer.enqueue_email_confirmation(user) end)\n  |> Repo.transaction()\n  |> case do\n    {:ok, %{user: user}} ->\n      {:ok, user}\n    {:error, _failed_operation, failed_value, _changes_so_far} ->\n      {:error, failed_value}\n  end\nend\n```\n\nBut this can be simplified with `transact/2`.\n\n```elixir\ndef register_user(params) do\n  Repo.transact(fn ->\n    with {:ok, user} <- Accounts.create_user(params),\n         {:ok, _log} <- Logs.log_action(:user_registered, user),\n         {:ok, _job} <- Mailer.enqueue_email_confirmation(user) do\n      {:ok, user}\n    end\n  end)\nend\n```\n\nIn Saša's own [words][1]:\n\n> I wrote `Repo.transact` after seeing a lot of production code along the lines of what's\n> written in that excellent [blog post][2] by @tomkonidas.\n>\n> The value proposition of `Repo.transact` is that control flow features such as passing\n> data around, branching, early exit, can be implemented with standard Elixir features,\n> such as variables, functions, and the `with` expression. The transactional logic is\n> less special, and it doesn't rely on some implicit behaviour of a function from some\n> library.\n>\n> Combined with the provable fact that the `transact` code is shorter (often\n> significantly), even in such simple example as in that blog post, I have no doubt that\n> the `transact` version is simpler and clearer.\n>\n> That's not to say that `Multi` is universally bad. The ability to provide each db\n> operation as data is definitely interesting, and could be useful in the cases where\n> the transactional steps need to be assembled dynamically (perhaps provided by the\n> client code). But in the vast majority of cases I've encountered, I find the multi\n> code needlessly difficult to read. This is true even in simple cases, and it becomes\n> progressively worse if the transactional logic is more involved (e.g. if it requires\n> branching early on in the transaction).\n>\n> Hence, I strongly prefer `transact`, and it's what I advise using in most situations.\n\n[1]: https://elixirforum.com/t/seeking-thoughts-on-advantages-of-the-repo-transact-pattern-vs-disadvantages-i-ve-read-about-ecto-multi/61733/2\n[2]: https://tomkonidas.com/repo-transact/\n[3]: https://hexdocs.pm/ecto/Ecto.Repo.html#c:transaction/2","ref":"KineticEcto.RepoTransact.html"},{"type":"macro","title":"KineticEcto.RepoTransact.__using__/1","doc":"Adds `Repo.transact/2`.","ref":"KineticEcto.RepoTransact.html#__using__/1"},{"type":"function","title":"KineticEcto.RepoTransact.transact/3","doc":"Runs the given function inside a transaction for the provided Ecto repo.\n\nThis function is a wrapper around `Ecto.Repo.transaction/2`, with the following\ndifferences:\n\n- It accepts only a lambda of arity 0 or 1 (i.e. it doesn't work with `Ecto.Multi`).\n- If the lambda returns `:ok | {:ok, result}` the transaction is committed.\n- If the lambda returns `:error | {:error, reason}` the transaction is rolled back.\n- If the lambda returns any other kind of result, an exception is raised, and the\n  transaction is rolled back.\n- The result of `transact` is the value returned by the lambda.\n\nThis function accepts the same options as [`Ecto.Repo.transaction/2`][1].\n\n[1]: https://hexdocs.pm/ecto/Ecto.Repo.html#c:transaction/2\n\n> #### Future Incompatibility {: .error}\n>\n> Ecto 3.13 or later defines `Repo.transact/2` to replace `Repo.transaction/2`. This is\n> mostly good, but importantly it works _differently_ than the version of `transact/2`\n> defined here in that it works with Ecto.Multi and the function version requires\n> `{:ok, result}` and `{:error, reason}` responses and does not work with `:ok` and\n> `:error` responses.","ref":"KineticEcto.RepoTransact.html#transact/3"},{"type":"module","title":"KineticEcto.Transact","doc":"This module has been renamed to `KineticEcto.RepoTransact` and will be removed in the\nnext major release.","ref":"KineticEcto.Transact.html"},{"type":"extras","title":"KineticEcto: Extensions for Ecto from Kinetic Cafe","doc":"# KineticEcto: Extensions for Ecto from Kinetic Cafe\n\n- code ::  \n- issues ::  \n\nKineticEcto is a collection of Ecto extensions previously used at Kinetic Cafe\nand Kinetic Commerce originally open sourced at [elixir-utilities][utils].","ref":"readme.html"},{"type":"extras","title":"Installation - KineticEcto: Extensions for Ecto from Kinetic Cafe","doc":"KineticEcto can be installed by adding `kinetic_ecto` to your list of\ndependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kinetic_ecto, \"~> 1.2\"}\n  ]\nend\n```\n\nDocumentation is found on [HexDocs][docs].","ref":"readme.html#installation"},{"type":"extras","title":"Semantic Versioning - KineticEcto: Extensions for Ecto from Kinetic Cafe","doc":"`KineticEcto` follows [Semantic Versioning 2.0][semver].\n\n[docs]: https://hexdocs.pm/kinetic_ecto\n[semver]: http://semver.org/\n[utils]: https://github.com/KineticCafe/elixir-utilities","ref":"readme.html#semantic-versioning"},{"type":"extras","title":"Contributing","doc":"# Contributing\n\nI value contributions to Enviable--bug reports, discussions, feature requests,\nand code contributions. New features should be proposed and discussed in an\n[issue][issues].\n\nBefore contributing patches, please read the [Licence](./LICENCE.md).\n\nEnviable is governed under the [Contributor Covenant Code of Conduct][cccoc].","ref":"CONTRIBUTING.md.html"},{"type":"extras","title":"Code Guidelines - Contributing","doc":"I have several guidelines to contributing code through pull requests:\n\n- All code changes require tests. In most cases, this will be added or updated\n  unit tests. I use [ExUnit][ExUnit].\n\n- I use code formatters, static analysis tools, and linting to ensure consistent\n  styles and formatting. There should be no warnings output from compile or test\n  run processes. I use `mix compile --warnings-as-errors`, [Credo][Credo], and\n  `mix format` (with [Styler][Styler])\n\n- Proposed changes should be on a thoughtfully-named topic branch and organized\n  into logical commit chunks as appropriate.\n\n- Use [Conventional Commits][conventional] with our\n  [conventions](#commit-conventions).\n\n- Versions must not be updated in pull requests.\n\n- Documentation should be added or updated as appropriate for new or updated\n  functionality.\n\n- New dependencies are discouraged and their addition must be discussed,\n  regardless whether it is a development dependency, optional dependency, or\n  runtime dependency.\n\n- All GitHub Actions checks marked as required must pass before a pull request\n  may be accepted and merged.","ref":"CONTRIBUTING.md.html#code-guidelines"},{"type":"extras","title":"Commit Conventions - Contributing","doc":"Enviable has adopted a variation of the Conventional Commits format for commit\nmessages. The following types are permitted:\n\n| Type    | Purpose                                               |\n| ------- | ----------------------------------------------------- |\n| `feat`  | A new feature                                         |\n| `fix`   | A bug fix                                             |\n| `chore` | A code change that is neither a bug fix nor a feature |\n| `docs`  | Documentation updates                                 |\n| `deps`  | Dependency updates, including GitHub Actions.         |\n\nI encourage the use of [Tim Pope's][tpope-qcm] or [Chris Beam's][cbeams]\nguidelines on the writing of commit messages\n\nI require the use of [git][trailers1] [trailers][trailers2] for specific\nadditional metadata and strongly encourage it for others. The conditionally\nrequired metadata trailers are:\n\n- `Breaking-Change`: if the change is a breaking change. **Do not** use the\n  shorthand form (`feat!(scope)`) or `BREAKING CHANGE`.\n\n- `Signed-off-by`: this is required for all developers except me, as outlined in\n  the [Licence](./LICENCE.md#developer-certificate-of-origin).\n\n- `Fixes` or `Resolves`: If a change fixes one or more open [issues][issues],\n  that issue must be included in the `Fixes` or `Resolves` trailer. Multiple\n  issues should be listed comma separated in the same trailer:\n  `Fixes: #1, #5, #7`, but _may_ appear in separate trailers. While both `Fixes`\n  and `Resolves` are synonyms, only _one_ should be used in a given commit or\n  pull request.\n\n- `Related to`: If a change does not fix an issue, those issue references should\n  be included in this trailer.","ref":"CONTRIBUTING.md.html#commit-conventions"},{"type":"extras","title":"Contributors - Contributing","doc":"Austin Ziegler created Enviable.\n\n[cccoc]: ./CODE_OF_CONDUCT.md\n[conventional]: https://www.conventionalcommits.org/en/v1.0.0/\n[credo]: https://github.com/rrrene/credo\n[styler]: https://github.com/adobe/styler\n[exunit]: https://hexdocs.pm/ex_unit/ExUnit.html\n[trailers1]: https://git-scm.com/docs/git-interpret-trailers\n[trailers2]: https://git-scm.com/docs/git-commit#Documentation/git-commit.txt---trailerlttokengtltvaluegt\n[issues]: https://github.com/halostatue/enviable/issues\n[tpope-qcm]: http://tbaggery.com/2008/04/19/a-note-about-git-commit-messages.html\n[cbeams]: https://cbea.ms/git-commit/","ref":"CONTRIBUTING.md.html#contributors"},{"type":"extras","title":"Code of Conduct","doc":"# Contributor Covenant Code of Conduct","ref":"CODE_OF_CONDUCT.md.html"},{"type":"extras","title":"Our Pledge - Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, caste, color, religion, or sexual\nidentity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"CODE_OF_CONDUCT.md.html#our-pledge"},{"type":"extras","title":"Our Standards - Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n- Demonstrating empathy and kindness toward other people\n- Being respectful of differing opinions, viewpoints, and experiences\n- Giving and gracefully accepting constructive feedback\n- Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n- Focusing on what is best not just for us as individuals, but for the overall\n  community\n\nExamples of unacceptable behavior include:\n\n- The use of sexualized language or imagery, and sexual attention or advances of\n  any kind\n- Trolling, insulting or derogatory comments, and personal or political attacks\n- Public or private harassment\n- Publishing others' private information, such as a physical or email address,\n  without their explicit permission\n- Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"CODE_OF_CONDUCT.md.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"CODE_OF_CONDUCT.md.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official email address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"CODE_OF_CONDUCT.md.html#scope"},{"type":"extras","title":"Enforcement - Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement at [INSERT CONTACT\nMETHOD]. All complaints will be reviewed and investigated promptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"CODE_OF_CONDUCT.md.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"CODE_OF_CONDUCT.md.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"CODE_OF_CONDUCT.md.html#1-correction"},{"type":"extras","title":"2. Warning - Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of\nactions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or permanent\nban.","ref":"CODE_OF_CONDUCT.md.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"CODE_OF_CONDUCT.md.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior, harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the\ncommunity.","ref":"CODE_OF_CONDUCT.md.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.1, available at\n .\n\nCommunity Impact Guidelines were inspired by\n[Mozilla's code of conduct enforcement ladder][Mozilla CoC].\n\nFor answers to common questions about this code of conduct, see the FAQ at\n . Translations are available at\n .\n\n[homepage]: https://www.contributor-covenant.org\n[Mozilla CoC]: https://github.com/mozilla/diversity","ref":"CODE_OF_CONDUCT.md.html#attribution"},{"type":"extras","title":"CHANGELOG","doc":"# KineticEcto Changelog","ref":"CHANGELOG.md.html"},{"type":"extras","title":"1.2.0 / 2025-06-30 - CHANGELOG","doc":"- Disabled `use KineticEcto.RepoTransact` if Ecto 3.13 or later is installed, as\n  it prevents compiling as the Ecto implementation of `Repo.transact/2` is not\n  marked `overridable` and causes build failures.\n\n  Unit tests for `KineticEcto.RepoTransact` were also removed.","ref":"CHANGELOG.md.html#1-2-0-2025-06-30"},{"type":"extras","title":"1.1.1 / 2025-02-03 - CHANGELOG","doc":"- Renamed `KineticEcto.Transact` to `KineticEcto.RepoTransact`.\n  `use KineticEcto.Transact` will now output a warning indicating that it has\n  been replaced with `use KineticEcto.RepoTransact`. The functionality otherwise\n  remains the same.","ref":"CHANGELOG.md.html#1-1-1-2025-02-03"},{"type":"extras","title":"1.1.0 / 2025-01-29 - CHANGELOG","doc":"- Added `KineticEcto.Transact`, an easy way to add Saša Juriċ's\n  `Repo.transact/2` to your arsenal.","ref":"CHANGELOG.md.html#1-1-0-2025-01-29"},{"type":"extras","title":"1.0.0 / 2025-01-19 - CHANGELOG","doc":"- Initial release, including changeset validation extensions","ref":"CHANGELOG.md.html#1-0-0-2025-01-19"},{"type":"extras","title":"Licence","doc":"# Licence\n\nKineticEcto is copyright © 2025 by Austin Ziegler.","ref":"LICENCE.md.html"},{"type":"extras","title":"Developer Certificate of Origin - Licence","doc":"All contributors **must** certify they are willing and able to provide their\ncontributions under the terms of this project's licences with the certification\nof the [Developer Certificate of Origin (Version 1.1)](licences/dco.txt).\n\nSuch certification is provided by ensuring that a `Signed-off-by`\n[commit trailer][trailer] is present on every commit:\n\n    Signed-off-by: FirstName LastName  \n\nThe `Signed-off-by` trailer can be automatically added by git with the `-s` or\n`--signoff` option on `git commit`:\n\n```sh\ngit commit --signoff\n```","ref":"LICENCE.md.html#developer-certificate-of-origin"},{"type":"extras","title":"Acknowledgements - Licence","doc":"- KineticEcto is based on code copyright © 2017-2024 by Kinetic Commerce,\n  licensed under the [Apache License, version 2.0](licences/APACHE-2.0.txt).\n\n  The following modules are based on [KineticCafe/elixir-utilities][utils] by\n  Kinetic Commerce:\n\n  - `Kinetic.Ecto`\n  - `Kinetic.Ecto.ChangsetValidations`\n\n- `Kinetic.Transact` is a near copy of `Repo.transact/2` in\n  [sasa1977/mix\\_phx\\_alt][mpa] by Saša Jurić, copyright © 2022-2024, licensed\n  under the MIT license.\n\n  > Copyright 2022, Saša Jurić\n  >\n  > Permission is hereby granted, free of charge, to any person obtaining a copy\n  > of this software and associated documentation files (the \"Software\"), to\n  > deal in the Software without restriction, including without limitation the\n  > rights to use, copy, modify, merge, publish, distribute, sublicense, and/or\n  > sell copies of the Software, and to permit persons to whom the Software is\n  > furnished to do so, subject to the following conditions:\n  >\n  > The above copyright notice and this permission notice shall be included in\n  > all copies or substantial portions of the Software.\n  >\n  > THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n  > IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n  > FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n  > AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n  > LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n  > FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS\n  > IN THE SOFTWARE.\n\n- Most of the SQLite unit test setup is based on\n  [elixir-sqlite/ecto\\_sqlite3][ectosqlite3], copyright © 2021–2025 Matthew A.\n  Johnston, licensed under the MIT license.\n\n  > Copyright (c) 2021 Matthew A. Johnston\n  >\n  > Permission is hereby granted, free of charge, to any person obtaining a copy\n  > of this software and associated documentation files (the \"Software\"), to\n  > deal in the Software without restriction, including without limitation the\n  > rights to use, copy, modify, merge, publish, distribute, sublicense, and/or\n  > sell copies of the Software, and to permit persons to whom the Software is\n  > furnished to do so, subject to the following conditions:\n  >\n  > The above copyright notice and this permission notice shall be included in\n  > all copies or substantial portions of the Software.\n  >\n  > THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n  > IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n  > FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n  > AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n  > LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING\n  > FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS\n  > IN THE SOFTWARE.\n\n[trailer]: https://git-scm.com/docs/git-interpret-trailers\n[utils]: https://github.com/KineticCafe/elixir-utilities\n[mpa]: https://github.com/sasa1977/mix_phx_alt/blob/d33a67fd6b2fa0ace5b6206487e774ef7a22ce5a/lib/core/repo.ex#L6-L44\n[ectosqlite3]: https://github.com/elixir-sqlite/ecto_sqlite3","ref":"LICENCE.md.html#acknowledgements"},{"type":"extras","title":"Apache License, version 2.0","doc":"Apache License\nVersion 2.0, January 2004\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n1. Definitions.\n\n\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.\n\n2. Grant of Copyright License.\n\nSubject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.\n\n3. Grant of Patent License.\n\nSubject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including\na cross-claim or counterclaim in a lawsuit) alleging that the Work or\na Contribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.\n\n4. Redistribution.\n\nYou may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n- You must give any other recipients of the Work or Derivative Works a copy of\n  this License; and\n- You must cause any modified files to carry prominent notices stating that You\n  changed the files; and\n- You must retain, in the Source form of any Derivative Works that You\n  distribute, all copyright, patent, trademark, and attribution notices from the\n  Source form of the Work, excluding those notices that do not pertain to any\n  part of the Derivative Works; and\n- If the Work includes a \"NOTICE\" text file as part of its distribution, then\n  any Derivative Works that You distribute must include a readable copy of the\n  attribution notices contained within such NOTICE file, excluding those notices\n  that do not pertain to any part of the Derivative Works, in at least one of\n  the following places: within a NOTICE text file distributed as part of the\n  Derivative Works; within the Source form or documentation, if provided along\n  with the Derivative Works; or, within a display generated by the Derivative\n  Works, if and wherever such third-party notices normally appear. The contents\n  of the NOTICE file are for informational purposes only and do not modify the\n  License. You may add Your own attribution notices within Derivative Works that\n  You distribute, alongside or as an addendum to the NOTICE text from the Work,\n  provided that such additional attribution notices cannot be construed as\n  modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.\n\n5. Submission of Contributions.\n\nUnless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.\n\n6. Trademarks.\n\nThis License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.\n\n7. Disclaimer of Warranty.\n\nUnless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE,\nNON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.\n\n8. Limitation of Liability.\n\nIn no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.\n\n9. Accepting Warranty or Additional Liability.\n\nWhile redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS","ref":"APACHE-2.0.txt.html"},{"type":"extras","title":"Developer Certificate of Origin","doc":"Developer Certificate of Origin\nVersion 1.1\n\nCopyright (C) 2004, 2006 The Linux Foundation and its contributors.\n\nEveryone is permitted to copy and distribute verbatim copies of this\nlicense document, but changing it is not allowed.\n\n\nDeveloper's Certificate of Origin 1.1\n\nBy making a contribution to this project, I certify that:\n\n(a) The contribution was created in whole or in part by me and I\n    have the right to submit it under the open source license\n    indicated in the file; or\n\n(b) The contribution is based upon previous work that, to the best\n    of my knowledge, is covered under an appropriate open source\n    license and I have the right under that license to submit that\n    work with modifications, whether created in whole or in part\n    by me, under the same open source license (unless I am\n    permitted to submit under a different license), as indicated\n    in the file; or\n\n(c) The contribution was provided directly to me by some other\n    person who certified (a), (b) or (c) and I have not modified\n    it.\n\n(d) I understand and agree that this project and the contribution\n    are public and that a record of the contribution (including all\n    personal information I submit with it, including my sign-off) is\n    maintained indefinitely and may be redistributed consistent with\n    this project or the open source license(s) involved.","ref":"dco.txt.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}